//a program to verify the main result of the paper
//"Ramsey-type cnstructions for arrangements of segments" by Jan Kyncl

//the program implements the algorithm described in the paper

#include <stdio.h>

//******** here choose the type of the arrangement that is expected on the input **********
#define CASE_KL 24           
//#define CASE_KL 42
//***********************************************************************************

//***** constants and macros *****
#define NUMBER_OF_SEGMENTS 13 //number of segments in one arrangement
#define MAXDEG 20             //upper bound for the degree of polynomials used in the calculations

#define TRUE 1
#define FALSE 0

#define max(a,b) (((a)>(b)) ? (a) : (b) )

//***** types *****
typedef struct {              //representation of a polynomial
  int degree;
  int a[MAXDEG+1];  //a[0..MAXDEG]
} POLYNOMIAL;

typedef POLYNOMIAL type_array[NUMBER_OF_SEGMENTS+1];  //a sequence of polynomials


//***** variables *****
FILE* f; //input file
FILE* g; //output file
type_array lx,ly,rx,ry;       //a sequence of x-coordinates of the left endpoints, etc.
POLYNOMIAL *a1,*b1,*c1,*a2,*b2,*c2,*sum1,*sum2,*sum3,*sum4,*temp_a,*temp_b,*temp_c;
unsigned char intersect[NUMBER_OF_SEGMENTS+1][NUMBER_OF_SEGMENTS+1]; 
unsigned char they_cross;     //temporary boolean variable storing the result of "s_i and s_j cross"
int errors;                   //number of pairs of segments that don't match with the prescribed intersection graph
int i,j;                      //auxiliary variables


//***** prototypes *****
void Load(void);              //loads the input from the file "input.txt"
unsigned char ShouldCross(int i, int j);  //defines the desired intersection graph 
POLYNOMIAL* New_polynomial(void);                     // creates new polynomial
POLYNOMIAL* sum(POLYNOMIAL *p, POLYNOMIAL *q);        // polynomial arithmetics
POLYNOMIAL* difference(POLYNOMIAL *p, POLYNOMIAL *q); 
POLYNOMIAL* minus(POLYNOMIAL *p);             
POLYNOMIAL* product(POLYNOMIAL *p, POLYNOMIAL *q);  
unsigned char is_nonnegative(POLYNOMIAL *p);   //tests whether p>=0 as x-->0
unsigned char is_nonpositive(POLYNOMIAL *p);   //tests whether p<=0 as x-->0

void WritePolynomials(void);                   //outputs the polynomials in more "mathematical" form
void WriteTerm(int a, int p);
void WriteFirstTerm(int a, int p);


//***** main function *****
int main(int argc, char *argv[])
{ 

  for (i=1;i<=NUMBER_OF_SEGMENTS;i++) { //initialization of the polynomials to 0
    lx[i].degree=-1;
    rx[i].degree=-1;
    ly[i].degree=-1;
    ry[i].degree=-1;
    for (j=0;j<=MAXDEG;j++) {
      lx[i].a[j]=0;
      rx[i].a[j]=0;
      ly[i].a[j]=0;
      ry[i].a[j]=0;
    }   //next j  
  }   // next i
  
  Load();
      
  errors=0;
  for (i=1;i<=NUMBER_OF_SEGMENTS;i++)
  for (j=i+1;j<=NUMBER_OF_SEGMENTS;j++) {    

    a1=difference( &(ry[i]),&(ly[i]) );
    a2=difference( &(ry[j]),&(ly[j]) );
    b1=difference( &(lx[i]),&(rx[i]) );
    b2=difference( &(lx[j]),&(rx[j]) );
    temp_a=product(&(rx[i]),&(ly[i]));
    temp_b=product(&(lx[i]),&(ry[i]));
    c1=difference(temp_a,temp_b);
    free(temp_a); free(temp_b);
    temp_a=product(&(rx[j]),&(ly[j]));
    temp_b=product(&(lx[j]),&(ry[j]));
    c2=difference(temp_a,temp_b);
    free(temp_a); free(temp_b);
    
    temp_a=product(a1,&(lx[j]));
    temp_b=product(b1,&(ly[j]));
    temp_c=sum(temp_a,temp_b);
    sum1=sum(temp_c,c1);
    free(temp_a); free(temp_b); free(temp_c);
    temp_a=product(a1,&(rx[j]));
    temp_b=product(b1,&(ry[j]));
    temp_c=sum(temp_a,temp_b);
    sum2=sum(temp_c,c1);
    free(temp_a); free(temp_b); free(temp_c);
    temp_a=product(a2,&(lx[i]));
    temp_b=product(b2,&(ly[i]));
    temp_c=sum(temp_a,temp_b);
    sum3=sum(temp_c,c2);
    free(temp_a); free(temp_b); free(temp_c);
    temp_a=product(a2,&(rx[i]));
    temp_b=product(b2,&(ry[i]));
    temp_c=sum(temp_a,temp_b);
    sum4=sum(temp_c,c2);
    free(temp_a); free(temp_b); free(temp_c);
  
    //instead of computing the polynomials d_{i,j}, we test each of the factors separately
    if (  ( (is_nonnegative(sum1)&&is_nonpositive(sum2)) || (is_nonpositive(sum1)&&is_nonnegative(sum2)) )
        &&( (is_nonnegative(sum3)&&is_nonpositive(sum4)) || (is_nonpositive(sum3)&&is_nonnegative(sum4)) ) )
      they_cross=TRUE;
    else they_cross=FALSE;

    if (ShouldCross(i,j))
      if (they_cross) printf("%i %i  X  ok\n",i,j);
      else {errors++; printf("error: %i %i do not cross\n",i,j);}
    else if (they_cross) {errors++; printf ("error: %i %i cross\n",i,j);}
         else printf("%i %i  || ok\n",i,j); 
  } //next j, next i
  printf("%i errors in total\n",errors);
  if (errors==0) printf("The construction is correct!\n");

  WritePolynomials();
} //end of main


//**************** functions *******************

void Load(void)
{
  FILE* f;
  int i,j,c;
  
#if (CASE_KL==24)
  if ( (f=fopen("input_24.txt", "r")) == NULL) {
    printf("File input_24.txt does not exist.");
    exit(-1);
  }   
#elif (CASE_KL==42)
  if ( (f=fopen("input_42.txt", "r")) == NULL) {
    printf("File input_42.txt does not exist.");
    exit(-1);
  }   
#else
  if ( (f=fopen("input.txt", "r")) == NULL) {
    printf("File input.txt does not exist.");
    exit(-1);
  }   
#endif /*CASE_KL*/
  printf("loading input.txt .../n");
  if (c=getc(f) != 'l') {printf("l in lx is missing");exit(-1);}
  if (c=getc(f) != 'x') {printf("x in lx is missing");exit(-1);}
  while (c=getc(f), (c != '\n')&&(c != EOF)); //readln
  for (i=1;i<=NUMBER_OF_SEGMENTS;i++) {
    if (fscanf(f,"%i",&lx[i].degree)<=0) {printf("error reading degree lx[%i]",i);exit(-1);}
    if (c=getc(f) != ',') {printf("',' missing in lx[%i]",i);exit(-1);}
    for (j=0;j<=lx[i].degree;j++)
      if (fscanf(f,"%i",&lx[i].a[j])<=0) {printf("error reading lx[%i][%j]",i,j);exit(-1);}
    while (c=getc(f), (c != '\n')&&(c != EOF)); //readln
  }
  if (c=getc(f) != 'l') {printf("l in ly is missing");exit(-1);}
  if (c=getc(f) != 'y') {printf("y in ly is missing");exit(-1);}
  while (c=getc(f), (c != '\n')&&(c != EOF)); //readln
  for (i=1;i<=NUMBER_OF_SEGMENTS;i++) {
    if (fscanf(f,"%i",&ly[i].degree)<=0) {printf("error reading degree ly[%i]",i);exit(-1);}
    if (c=getc(f) != ',') {printf("',' missing in ly[%i]",i);exit(-1);}
    for (j=0;j<=ly[i].degree;j++)
      if (fscanf(f,"%i",&ly[i].a[j])<=0) {printf("error reading ly[%i][%j]",i,j);exit(-1);}
    while (c=getc(f), (c != '\n')&&(c != EOF)); //readln
  }
  if (c=getc(f) != 'r') {printf("r in rx is missing");exit(-1);}
  if (c=getc(f) != 'x') {printf("x in rx is missing");exit(-1);}
  while (c=getc(f), (c != '\n')&&(c != EOF)); //readln
  for (i=1;i<=NUMBER_OF_SEGMENTS;i++) {
    if (fscanf(f,"%i",&rx[i].degree)<=0) {printf("error reading degree rx[%i]",i);exit(-1);}
    if (c=getc(f) != ',') {printf("',' missing in rx[%i]",i);exit(-1);}
    for (j=0;j<=rx[i].degree;j++)
      if (fscanf(f,"%i",&rx[i].a[j])<=0) {printf("error reading rx[%i][%j]",i,j);exit(-1);}
    while (c=getc(f), (c != '\n')&&(c != EOF)); //readln
  }
  if (c=getc(f) != 'r') {printf("r in ry is missing");exit(-1);}
  if (c=getc(f) != 'y') {printf("y in ry is missing");exit(-1);}
  while (c=getc(f), (c != '\n')&&(c != EOF)); //readln
  for (i=1;i<=NUMBER_OF_SEGMENTS;i++) {
    if (fscanf(f,"%i",&ry[i].degree)<=0) {printf("error reading degree ry[%i]",i);exit(-1);}
    if (c=getc(f) != ',') {printf("',' missing in ry[%i]",i);exit(-1);}
    for (j=0;j<=ry[i].degree;j++)
      if (fscanf(f,"%i",&ry[i].a[j])<=0) {printf("error reading ry[%i][%j]",i,j);exit(-1);}
    while (c=getc(f), (c != '\n')&&(c != EOF)); //readln
  }
  printf("loading complete/n");
  fclose(f);
} //end of Load


unsigned char ShouldCross(int i, int j)  //defines the intersection graph; assumes i<j 
{
#if (CASE_KL==24)
  if (j<=i) {printf("i should be smaller than j when calling ShouldCross(i,j)\n");exit(-1);}
  if ((j-i==1)||(j-i==5)||(j-i==8)||(j-i==12)) return(TRUE); //Cayley graph Cay(Z_13; 1, 5)
  else return(FALSE);
#elif (CASE_KL==42)
  if (j<=i) {printf("i should be smaller than j when calling ShouldCross(i,j)\n");exit(-1);}
  if ((j-i==1)||(j-i==5)||(j-i==8)||(j-i==12)) return(FALSE); //complement of Cay(Z_13; 1, 5)
  else return(TRUE);
#else
  return(FALSE); 
#endif /*CASE_KL*/
} //end of ShouldCross


//------some basic operations with polynomials with integer coefficients------

POLYNOMIAL* New_polynomial(void) 
{
  int i;
  POLYNOMIAL *r;

  if ((r=(POLYNOMIAL*)malloc(sizeof(POLYNOMIAL)) )==NULL) {printf("not enough memory to create new polynomial."); exit(-1);}
  r->degree=-1;
  for (i=0;i<=MAXDEG;i++) r->a[i]=0;
  return(r);
} //end of New_polynomial 

POLYNOMIAL* sum(POLYNOMIAL *p, POLYNOMIAL *q)
{
  int i,maxd;
  POLYNOMIAL *r;
  
  r=New_polynomial();
  maxd=max(p->degree,q->degree);
  for(i=0;i<=maxd;i++) r->a[i] = p->a[i] + q->a[i];
    //now we compute the degree of the result
  i=maxd;
  while ((i>=0)&&(r->a[i]==0)) i--;   //searching first non-zero term
  r->degree=i;
  return(r);
} //end of sum

POLYNOMIAL* difference(POLYNOMIAL *p, POLYNOMIAL *q)
{
  int i,maxd;
  POLYNOMIAL *r;
  
  r=New_polynomial();
  maxd=max(p->degree,q->degree);
  for(i=0;i<=maxd;i++) r->a[i] = p->a[i] - q->a[i];
   //now we compute the degree of the result
  i=maxd;
  while ((i>=0)&&(r->a[i]==0)) i--;   //searching first non-zero term
  r->degree=i;
  return(r);
} //end of difference

POLYNOMIAL* minus(POLYNOMIAL *p)  
{
  int i;
  POLYNOMIAL* r;

  r=New_polynomial();
  r->degree = p->degree;
  for(i=0;i<=p->degree;i++) r->a[i] = -p->a[i];
  return(r);
} //end of minus

POLYNOMIAL* product(POLYNOMIAL *p, POLYNOMIAL *q) 
//straighforward quadratic-time algorithm for multiplication
{
  int i,j,totaldeg;
  POLYNOMIAL* r;

  r=New_polynomial();
  if ((p->degree==-1)||(q->degree==-1)) {  //multiplying by zero
    return(r);
  }
  else {
    totaldeg = p->degree + q->degree;
    if (totaldeg>MAXDEG) {printf("degree too high"); exit(-1);}
    r->degree=totaldeg;
    for (i=0;i<=p->degree;i++)
    for (j=0;j<=q->degree;j++)
      r->a[i+j] += (p->a[i]*q->a[j]);  
    return(r);
  } //end of else
} //end of product

unsigned char is_nonnegative(POLYNOMIAL *p)
{
  int i;
  
  if (p->degree==-1) 
    return(TRUE);
  else {
    i=0;
    while (p->a[i]==0) i++; //searching last non-zero term 
    if (p->a[i] > 0) return(TRUE); else return(FALSE);
  }
} //end of is_nonnegative

unsigned char is_nonpositive(POLYNOMIAL *p)
{
  int i;
  
  if (p->degree==-1) 
    return(TRUE);
  else {
    i=0;
    while (p->a[i]==0) i++; //searching last non-zero term 
    if (p->a[i] < 0) return(TRUE); else return(FALSE);
  }
} //end of is_nonpositive




void WritePolynomials(void)
{
  int i,j,c;
  
  if ( (g=fopen("mathpoly.txt", "w")) == NULL) {
    printf("Couldn't create mathpoly.txt.");
    exit(-1);
  }   
  for (i=1;i<=NUMBER_OF_SEGMENTS;i++) {
    fprintf(g,"%i  ",i); //row number
    //lx
    if (lx[i].degree==-1) fprintf(g,"0");
    else {
      j=0;
      while (lx[i].a[j]==0) j++;
      WriteFirstTerm(lx[i].a[j],j);
      for (j=j+1;j<=lx[i].degree;j++) 
        if (lx[i].a[j]!=0) WriteTerm(lx[i].a[j],j); 
    } //end of else 
    fprintf(g,",  "); //separates polynomials
    //ly
    if (ly[i].degree==-1) fprintf(g,"0");
    else {
      j=0;
      while (ly[i].a[j]==0) j++;
      WriteFirstTerm(ly[i].a[j],j);
      for (j=j+1;j<=ly[i].degree;j++) 
        if (ly[i].a[j]!=0) WriteTerm(ly[i].a[j],j); 
    } //end of else
    fprintf(g,",  "); //separates polynomials
    //rx
    if (rx[i].degree==-1) fprintf(g,"0");
    else {
      j=0;
      while (rx[i].a[j]==0) j++;
      WriteFirstTerm(rx[i].a[j],j);
      for (j=j+1;j<=rx[i].degree;j++) 
        if (rx[i].a[j]!=0) WriteTerm(rx[i].a[j],j); 
    } //end of else
    fprintf(g,",  "); //separates polynomials
    //ry
    if (ry[i].degree==-1) fprintf(g,"0");
    else {
      j=0;
      while (ry[i].a[j]==0) j++;
      WriteFirstTerm(ry[i].a[j],j);
      for (j=j+1;j<=ry[i].degree;j++) 
        if (ry[i].a[j]!=0) WriteTerm(ry[i].a[j],j); 
    } //end of else 
    fprintf(g,"\n"); 
  }
  fclose(g);
} //end of WritePolynomials



void WriteTerm(int a, int p) //writes a term with non-zero coefficient 
{
  if (a>=0) fprintf(g,"+");
  if (p==0) fprintf(g,"%i",a);
  else if (a==-1) fprintf(g,"-");
       else if (a!=1) fprintf(g,"%i",a); 
  if (p==1) fprintf(g,"x");
  else if (p>=2) fprintf(g,"x^%i",p);
} //end of WriteTerm

void WriteFirstTerm(int a, int p) //writes first term with non-zero coefficient 
{
  if (p==0) fprintf(g,"%i",a);
  else if (a==-1) fprintf(g,"-");
       else if (a!=1) fprintf(g,"%i",a); 
  if (p==1) fprintf(g,"x");
  else if (p>=2) fprintf(g,"x^%i",p);
} //end of WriteFirstTerm

